# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: virtual-trends.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import common_models_pb2 as Protobuf_dot_common__models__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='virtual-trends.proto',
  package='Bsii.SolarFocus.Protobuf.Trends',
  syntax='proto3',
  serialized_options=b'\252\002\034Bsii.SolarFocus.Lib.Protobuf',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x14virtual-trends.proto\x12\x1f\x42sii.SolarFocus.Protobuf.Trends\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cProtobuf/common-models.proto\x1a\x1egoogle/protobuf/wrappers.proto\"D\n\tDataPoint\x12(\n\x04time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05value\x18\x02 \x01(\x02\"\x94\x01\n\x11TagTimeSeriesData\x12>\n\x08trend_id\x18\x01 \x01(\x0b\x32,.Bsii.SolarFocus.Protobuf.TagsIdentifierBase\x12?\n\x0btime_series\x18\x02 \x03(\x0b\x32*.Bsii.SolarFocus.Protobuf.Trends.DataPoint\"\xca\x02\n\x14TimeSeriesQueryInput\x12G\n\x11trends_identifier\x18\x01 \x03(\x0b\x32,.Bsii.SolarFocus.Protobuf.TagsIdentifierBase\x12\x32\n\x0estart_date_utc\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0c\x65nd_date_utc\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12N\n\x12\x63\x61lculation_method\x18\x04 \x01(\x0e\x32\x32.Bsii.SolarFocus.Protobuf.Trends.CalculationMethod\x12\x33\n\x10sample_frequency\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\"\xa0\x01\n\x15TimeSeriesQueryResult\x12\x42\n\x10problematic_tags\x18\x01 \x03(\x0b\x32(.Bsii.SolarFocus.Protobuf.ProblematicTag\x12\x43\n\x07results\x18\x02 \x03(\x0b\x32\x32.Bsii.SolarFocus.Protobuf.Trends.TagTimeSeriesData\"\xa0\x01\n\rTrendMetadata\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12.\n\tmin_value\x18\x03 \x01(\x0b\x32\x1b.google.protobuf.FloatValue\x12.\n\tmax_value\x18\x04 \x01(\x0b\x32\x1b.google.protobuf.FloatValue\x12\x0c\n\x04unit\x18\x05 \x01(\t\"\x86\x03\n\x13VirtualTrendsModule\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08username\x18\x02 \x01(\t\x12\x12\n\nbase_image\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x13\n\x0b\x65nvironment\x18\x05 \x01(\t\x12N\n\x12\x63\x61lculation_method\x18\x06 \x01(\x0e\x32\x32.Bsii.SolarFocus.Protobuf.Trends.CalculationMethod\x12\x33\n\x10sample_frequency\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x45\n\x0cinput_trends\x18\x08 \x03(\x0b\x32/.Bsii.SolarFocus.Protobuf.VirtualModuleInputTag\x12\x45\n\routput_trends\x18\t \x03(\x0b\x32..Bsii.SolarFocus.Protobuf.Trends.TrendMetadata*\xae\x01\n\x11\x43\x61lculationMethod\x12\x08\n\x04NONE\x10\x00\x12\x16\n\x12LINEAR_INTERPOLATE\x10\x01\x12\x1c\n\x18\x41VERAGE_OVER_SAMPLE_FREQ\x10\x02\x12\x18\n\x14MAX_OVER_SAMPLE_FREQ\x10\x03\x12\x18\n\x14MIN_OVRR_SAMPLE_FREQ\x10\x04\x12\x11\n\rLEFT_NEIGHBOR\x10\x05\x12\x12\n\x0eRIGHT_NEIGHBOR\x10\x06\x32\x94\x01\n\x18VirtualTrendsGrpcService\x12x\n\x07GetData\x12\x35.Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryInput\x1a\x36.Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryResultB\x1f\xaa\x02\x1c\x42sii.SolarFocus.Lib.Protobufb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,Protobuf_dot_common__models__pb2.DESCRIPTOR,google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,])

_CALCULATIONMETHOD = _descriptor.EnumDescriptor(
  name='CalculationMethod',
  full_name='Bsii.SolarFocus.Protobuf.Trends.CalculationMethod',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NONE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LINEAR_INTERPOLATE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AVERAGE_OVER_SAMPLE_FREQ', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MAX_OVER_SAMPLE_FREQ', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MIN_OVRR_SAMPLE_FREQ', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LEFT_NEIGHBOR', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RIGHT_NEIGHBOR', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1458,
  serialized_end=1632,
)
_sym_db.RegisterEnumDescriptor(_CALCULATIONMETHOD)

CalculationMethod = enum_type_wrapper.EnumTypeWrapper(_CALCULATIONMETHOD)
NONE = 0
LINEAR_INTERPOLATE = 1
AVERAGE_OVER_SAMPLE_FREQ = 2
MAX_OVER_SAMPLE_FREQ = 3
MIN_OVRR_SAMPLE_FREQ = 4
LEFT_NEIGHBOR = 5
RIGHT_NEIGHBOR = 6



_DATAPOINT = _descriptor.Descriptor(
  name='DataPoint',
  full_name='Bsii.SolarFocus.Protobuf.Trends.DataPoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='Bsii.SolarFocus.Protobuf.Trends.DataPoint.time', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='Bsii.SolarFocus.Protobuf.Trends.DataPoint.value', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=184,
  serialized_end=252,
)


_TAGTIMESERIESDATA = _descriptor.Descriptor(
  name='TagTimeSeriesData',
  full_name='Bsii.SolarFocus.Protobuf.Trends.TagTimeSeriesData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='trend_id', full_name='Bsii.SolarFocus.Protobuf.Trends.TagTimeSeriesData.trend_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time_series', full_name='Bsii.SolarFocus.Protobuf.Trends.TagTimeSeriesData.time_series', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=255,
  serialized_end=403,
)


_TIMESERIESQUERYINPUT = _descriptor.Descriptor(
  name='TimeSeriesQueryInput',
  full_name='Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='trends_identifier', full_name='Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryInput.trends_identifier', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_date_utc', full_name='Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryInput.start_date_utc', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_date_utc', full_name='Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryInput.end_date_utc', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='calculation_method', full_name='Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryInput.calculation_method', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sample_frequency', full_name='Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryInput.sample_frequency', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=406,
  serialized_end=736,
)


_TIMESERIESQUERYRESULT = _descriptor.Descriptor(
  name='TimeSeriesQueryResult',
  full_name='Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='problematic_tags', full_name='Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryResult.problematic_tags', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='results', full_name='Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryResult.results', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=739,
  serialized_end=899,
)


_TRENDMETADATA = _descriptor.Descriptor(
  name='TrendMetadata',
  full_name='Bsii.SolarFocus.Protobuf.Trends.TrendMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Bsii.SolarFocus.Protobuf.Trends.TrendMetadata.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='Bsii.SolarFocus.Protobuf.Trends.TrendMetadata.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_value', full_name='Bsii.SolarFocus.Protobuf.Trends.TrendMetadata.min_value', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_value', full_name='Bsii.SolarFocus.Protobuf.Trends.TrendMetadata.max_value', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unit', full_name='Bsii.SolarFocus.Protobuf.Trends.TrendMetadata.unit', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=902,
  serialized_end=1062,
)


_VIRTUALTRENDSMODULE = _descriptor.Descriptor(
  name='VirtualTrendsModule',
  full_name='Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsModule',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsModule.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='username', full_name='Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsModule.username', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='base_image', full_name='Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsModule.base_image', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsModule.description', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='environment', full_name='Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsModule.environment', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='calculation_method', full_name='Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsModule.calculation_method', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sample_frequency', full_name='Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsModule.sample_frequency', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='input_trends', full_name='Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsModule.input_trends', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output_trends', full_name='Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsModule.output_trends', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1065,
  serialized_end=1455,
)

_DATAPOINT.fields_by_name['time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TAGTIMESERIESDATA.fields_by_name['trend_id'].message_type = Protobuf_dot_common__models__pb2._TAGSIDENTIFIERBASE
_TAGTIMESERIESDATA.fields_by_name['time_series'].message_type = _DATAPOINT
_TIMESERIESQUERYINPUT.fields_by_name['trends_identifier'].message_type = Protobuf_dot_common__models__pb2._TAGSIDENTIFIERBASE
_TIMESERIESQUERYINPUT.fields_by_name['start_date_utc'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TIMESERIESQUERYINPUT.fields_by_name['end_date_utc'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TIMESERIESQUERYINPUT.fields_by_name['calculation_method'].enum_type = _CALCULATIONMETHOD
_TIMESERIESQUERYINPUT.fields_by_name['sample_frequency'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_TIMESERIESQUERYRESULT.fields_by_name['problematic_tags'].message_type = Protobuf_dot_common__models__pb2._PROBLEMATICTAG
_TIMESERIESQUERYRESULT.fields_by_name['results'].message_type = _TAGTIMESERIESDATA
_TRENDMETADATA.fields_by_name['min_value'].message_type = google_dot_protobuf_dot_wrappers__pb2._FLOATVALUE
_TRENDMETADATA.fields_by_name['max_value'].message_type = google_dot_protobuf_dot_wrappers__pb2._FLOATVALUE
_VIRTUALTRENDSMODULE.fields_by_name['calculation_method'].enum_type = _CALCULATIONMETHOD
_VIRTUALTRENDSMODULE.fields_by_name['sample_frequency'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_VIRTUALTRENDSMODULE.fields_by_name['input_trends'].message_type = Protobuf_dot_common__models__pb2._VIRTUALMODULEINPUTTAG
_VIRTUALTRENDSMODULE.fields_by_name['output_trends'].message_type = _TRENDMETADATA
DESCRIPTOR.message_types_by_name['DataPoint'] = _DATAPOINT
DESCRIPTOR.message_types_by_name['TagTimeSeriesData'] = _TAGTIMESERIESDATA
DESCRIPTOR.message_types_by_name['TimeSeriesQueryInput'] = _TIMESERIESQUERYINPUT
DESCRIPTOR.message_types_by_name['TimeSeriesQueryResult'] = _TIMESERIESQUERYRESULT
DESCRIPTOR.message_types_by_name['TrendMetadata'] = _TRENDMETADATA
DESCRIPTOR.message_types_by_name['VirtualTrendsModule'] = _VIRTUALTRENDSMODULE
DESCRIPTOR.enum_types_by_name['CalculationMethod'] = _CALCULATIONMETHOD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DataPoint = _reflection.GeneratedProtocolMessageType('DataPoint', (_message.Message,), {
  'DESCRIPTOR' : _DATAPOINT,
  '__module__' : 'virtual_trends_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.Trends.DataPoint)
  })
_sym_db.RegisterMessage(DataPoint)

TagTimeSeriesData = _reflection.GeneratedProtocolMessageType('TagTimeSeriesData', (_message.Message,), {
  'DESCRIPTOR' : _TAGTIMESERIESDATA,
  '__module__' : 'virtual_trends_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.Trends.TagTimeSeriesData)
  })
_sym_db.RegisterMessage(TagTimeSeriesData)

TimeSeriesQueryInput = _reflection.GeneratedProtocolMessageType('TimeSeriesQueryInput', (_message.Message,), {
  'DESCRIPTOR' : _TIMESERIESQUERYINPUT,
  '__module__' : 'virtual_trends_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryInput)
  })
_sym_db.RegisterMessage(TimeSeriesQueryInput)

TimeSeriesQueryResult = _reflection.GeneratedProtocolMessageType('TimeSeriesQueryResult', (_message.Message,), {
  'DESCRIPTOR' : _TIMESERIESQUERYRESULT,
  '__module__' : 'virtual_trends_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryResult)
  })
_sym_db.RegisterMessage(TimeSeriesQueryResult)

TrendMetadata = _reflection.GeneratedProtocolMessageType('TrendMetadata', (_message.Message,), {
  'DESCRIPTOR' : _TRENDMETADATA,
  '__module__' : 'virtual_trends_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.Trends.TrendMetadata)
  })
_sym_db.RegisterMessage(TrendMetadata)

VirtualTrendsModule = _reflection.GeneratedProtocolMessageType('VirtualTrendsModule', (_message.Message,), {
  'DESCRIPTOR' : _VIRTUALTRENDSMODULE,
  '__module__' : 'virtual_trends_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsModule)
  })
_sym_db.RegisterMessage(VirtualTrendsModule)


DESCRIPTOR._options = None

_VIRTUALTRENDSGRPCSERVICE = _descriptor.ServiceDescriptor(
  name='VirtualTrendsGrpcService',
  full_name='Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsGrpcService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1635,
  serialized_end=1783,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetData',
    full_name='Bsii.SolarFocus.Protobuf.Trends.VirtualTrendsGrpcService.GetData',
    index=0,
    containing_service=None,
    input_type=_TIMESERIESQUERYINPUT,
    output_type=_TIMESERIESQUERYRESULT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_VIRTUALTRENDSGRPCSERVICE)

DESCRIPTOR.services_by_name['VirtualTrendsGrpcService'] = _VIRTUALTRENDSGRPCSERVICE

# @@protoc_insertion_point(module_scope)



#!/usr/bin/python3
from abc import ABC, abstractmethod
from datetime import datetime, timedelta
from typing import List, Dict
import numpy as np
import pandas as pd
import os
import types
import json
from . import solar_focus_trends_lib as sofoc
from common.models import VTCommandLineArgsBase
from common.api import virtual_trends_pb2
import google.protobuf.json_format as json_format
from common.api import common_models_pb2
from user_calculation import calculate as user_calc


class VirtualTrendRunner(ABC):
    def __init__(self, metadata_file: str):
        self.metadata = self.get_metadata(metadata_file)

    def calculate(self, start_date_utc: datetime, end_date_utc: datetime, inputs_from_solar_focus: pd.DataFrame = None) -> pd.DataFrame:
        res = user_calc(start_date_utc, end_date_utc, inputs_from_solar_focus)
        self.validate_res(res)
        return res
    
    def validate_res(self, res: pd.DataFrame):
        """
        Verify that result of this virtual trend match the defines outputs
        """
        expected = set([x.name for x in self.metadata.output_trends])
        results = set([x for x in res.columns])
        if len(expected) != len(results) or expected != results:
            raise Exception(
                "The calculated results map doesn't match the defined output tags: expected - {0}, received - {1}".format(expected, results))

    def execute(self, vtCommandLineArgs: VTCommandLineArgsBase, start_utc: datetime, end_utc: datetime) -> pd.DataFrame:
        """
        This is the executaion call for this VT calculation flow
        """
        query = sofoc.build_query( start_utc, end_utc,
            self.metadata.input_trends, self.metadata.calculation_method, self.metadata.sample_frequency)
        if len(query.trends_identifier) > 0:
            res:  List[TagTimeSeriesData] = sofoc.get_data(self.metadata.input_trends, query, vtCommandLineArgs)
            return self.calculate(query.start_date_utc.ToDatetime(), query.end_date_utc.ToDatetime(), res)
        else:
            return self.calculate(query.start_date_utc.ToDatetime(), query.end_date_utc.ToDatetime())

    def get_metadata(self, metadata_file: str) -> virtual_trends_pb2.VirtualTrendsModule:
        with open(metadata_file, 'r', encoding='utf-8') as f:
            message = virtual_trends_pb2.VirtualTrendsModule()
            json_format.Parse(f.read(), message)
            return message

    def df_to_tag_time_series_data_arr(self, res: pd.DataFrame) -> virtual_trends_pb2.TimeSeriesQueryResult:
        time_series_query_result = virtual_trends_pb2.TimeSeriesQueryResult()
        col_index=0
        for trend_metadata in self.metadata.output_trends:
            tag_time_series_data = virtual_trends_pb2.TagTimeSeriesData()
            tag_time_series_data.trend_id.tag_type = common_models_pb2.IdentifierTypes.VIRTUAL
            tag_time_series_data.trend_id.virtual_tag_identifier.module_name = self.metadata.name
            tag_time_series_data.trend_id.virtual_tag_identifier.output_name = trend_metadata.name
            for index, row in res.iterrows():
                data_point = virtual_trends_pb2.DataPoint()
                data_point.time.FromDatetime(index)
                data_point.value = row[col_index]
                tag_time_series_data.time_series.append(data_point)
            time_series_query_result.results.append(tag_time_series_data)
            col_index+=1
        return time_series_query_result



import pyodbc
import pandas as pd


class SqlReader:
    def __init__(self, connection_string: str):
        self.connection_string = connection_string

    def create_connection(self):
        return pyodbc.connect(self.connection_string)

    def get_df_from_db(self, query: str, sfo_column: str, date_format="%d/%m/%Y"):
        conn = self.create_connection()
        df = pd.read_sql(query, conn)
        # set date column as index column
        df.set_index(sfo_column, inplace=True)
        return df

    @staticmethod
    def create_sql_server_trusted_connection_string(server: str, db_name: str):
        return "Driver=SQL Server Native Client 11.0;Server={0};Database={1};Trusted_Connection=yes;".format(server, db_name)    
    @staticmethod
    def create_sql_server_connection_string(server: str, db_name: str, username: str, password: str):
        return "DRIVER=ODBC Driver 17 for SQL Server;SERVER={0};DATABASE={1};UID={2};PWD={3};".format(server, db_name, username, password)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: virtual-sfv.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_models_pb2 as Protobuf_dot_common__models__pb2
from . import solar_field_common_pb2 as Protobuf_dot_solar__field__common__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='virtual-sfv.proto',
  package='Bsii.SolarFocus.Protobuf.SolarField',
  syntax='proto3',
  serialized_options=b'\252\002\034Bsii.SolarFocus.Lib.Protobuf',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x11virtual-sfv.proto\x12#Bsii.SolarFocus.Protobuf.SolarField\x1a\x1cProtobuf/common-models.proto\x1a!Protobuf/solar-field-common.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x89\x02\n\x08SfvQuery\x12\x32\n\x0equery_time_utc\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12)\n\x06margin\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\x12U\n\x12\x63\x61lculation_method\x18\x03 \x01(\x0e\x32\x39.Bsii.SolarFocus.Protobuf.SolarField.SfvCalculationMethod\x12G\n\x11tags_descriptions\x18\x04 \x03(\x0b\x32,.Bsii.SolarFocus.Protobuf.TagsIdentifierBase\">\n\x0bTagMetadata\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0c\n\x04unit\x18\x03 \x01(\t\"\x88\x03\n\x10VirtualSfvModule\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08username\x18\x02 \x01(\t\x12\x12\n\nbase_image\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x13\n\x0b\x65nvironment\x18\x05 \x01(\t\x12U\n\x12\x63\x61lculation_method\x18\x06 \x01(\x0e\x32\x39.Bsii.SolarFocus.Protobuf.SolarField.SfvCalculationMethod\x12\x33\n\x10sample_frequency\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x43\n\ninput_tags\x18\x08 \x03(\x0b\x32/.Bsii.SolarFocus.Protobuf.VirtualModuleInputTag\x12\x45\n\x0boutput_tags\x18\t \x03(\x0b\x32\x30.Bsii.SolarFocus.Protobuf.SolarField.TagMetadata2\x85\x01\n\x15VirtualSfvGrpcService\x12l\n\x07GetData\x12-.Bsii.SolarFocus.Protobuf.SolarField.SfvQuery\x1a\x32.Bsii.SolarFocus.Protobuf.SolarField.SolarFieldDtoB\x1f\xaa\x02\x1c\x42sii.SolarFocus.Lib.Protobufb\x06proto3'
  ,
  dependencies=[Protobuf_dot_common__models__pb2.DESCRIPTOR,Protobuf_dot_solar__field__common__pb2.DESCRIPTOR,google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_SFVQUERY = _descriptor.Descriptor(
  name='SfvQuery',
  full_name='Bsii.SolarFocus.Protobuf.SolarField.SfvQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='query_time_utc', full_name='Bsii.SolarFocus.Protobuf.SolarField.SfvQuery.query_time_utc', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='margin', full_name='Bsii.SolarFocus.Protobuf.SolarField.SfvQuery.margin', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='calculation_method', full_name='Bsii.SolarFocus.Protobuf.SolarField.SfvQuery.calculation_method', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tags_descriptions', full_name='Bsii.SolarFocus.Protobuf.SolarField.SfvQuery.tags_descriptions', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=189,
  serialized_end=454,
)


_TAGMETADATA = _descriptor.Descriptor(
  name='TagMetadata',
  full_name='Bsii.SolarFocus.Protobuf.SolarField.TagMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Bsii.SolarFocus.Protobuf.SolarField.TagMetadata.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='Bsii.SolarFocus.Protobuf.SolarField.TagMetadata.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unit', full_name='Bsii.SolarFocus.Protobuf.SolarField.TagMetadata.unit', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=456,
  serialized_end=518,
)


_VIRTUALSFVMODULE = _descriptor.Descriptor(
  name='VirtualSfvModule',
  full_name='Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvModule',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvModule.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='username', full_name='Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvModule.username', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='base_image', full_name='Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvModule.base_image', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvModule.description', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='environment', full_name='Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvModule.environment', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='calculation_method', full_name='Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvModule.calculation_method', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sample_frequency', full_name='Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvModule.sample_frequency', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='input_tags', full_name='Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvModule.input_tags', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output_tags', full_name='Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvModule.output_tags', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=521,
  serialized_end=913,
)

_SFVQUERY.fields_by_name['query_time_utc'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SFVQUERY.fields_by_name['margin'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_SFVQUERY.fields_by_name['calculation_method'].enum_type = Protobuf_dot_solar__field__common__pb2._SFVCALCULATIONMETHOD
_SFVQUERY.fields_by_name['tags_descriptions'].message_type = Protobuf_dot_common__models__pb2._TAGSIDENTIFIERBASE
_VIRTUALSFVMODULE.fields_by_name['calculation_method'].enum_type = Protobuf_dot_solar__field__common__pb2._SFVCALCULATIONMETHOD
_VIRTUALSFVMODULE.fields_by_name['sample_frequency'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_VIRTUALSFVMODULE.fields_by_name['input_tags'].message_type = Protobuf_dot_common__models__pb2._VIRTUALMODULEINPUTTAG
_VIRTUALSFVMODULE.fields_by_name['output_tags'].message_type = _TAGMETADATA
DESCRIPTOR.message_types_by_name['SfvQuery'] = _SFVQUERY
DESCRIPTOR.message_types_by_name['TagMetadata'] = _TAGMETADATA
DESCRIPTOR.message_types_by_name['VirtualSfvModule'] = _VIRTUALSFVMODULE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SfvQuery = _reflection.GeneratedProtocolMessageType('SfvQuery', (_message.Message,), {
  'DESCRIPTOR' : _SFVQUERY,
  '__module__' : 'virtual_sfv_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.SolarField.SfvQuery)
  })
_sym_db.RegisterMessage(SfvQuery)

TagMetadata = _reflection.GeneratedProtocolMessageType('TagMetadata', (_message.Message,), {
  'DESCRIPTOR' : _TAGMETADATA,
  '__module__' : 'virtual_sfv_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.SolarField.TagMetadata)
  })
_sym_db.RegisterMessage(TagMetadata)

VirtualSfvModule = _reflection.GeneratedProtocolMessageType('VirtualSfvModule', (_message.Message,), {
  'DESCRIPTOR' : _VIRTUALSFVMODULE,
  '__module__' : 'virtual_sfv_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvModule)
  })
_sym_db.RegisterMessage(VirtualSfvModule)


DESCRIPTOR._options = None

_VIRTUALSFVGRPCSERVICE = _descriptor.ServiceDescriptor(
  name='VirtualSfvGrpcService',
  full_name='Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvGrpcService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=916,
  serialized_end=1049,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetData',
    full_name='Bsii.SolarFocus.Protobuf.SolarField.VirtualSfvGrpcService.GetData',
    index=0,
    containing_service=None,
    input_type=_SFVQUERY,
    output_type=Protobuf_dot_solar__field__common__pb2._SOLARFIELDDTO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_VIRTUALSFVGRPCSERVICE)

DESCRIPTOR.services_by_name['VirtualSfvGrpcService'] = _VIRTUALSFVGRPCSERVICE

# @@protoc_insertion_point(module_scope)



#!/usr/bin/python3
from abc import ABC, abstractmethod
from datetime import datetime, timedelta
from . import solar_focus_sfv_lib as sofoc
from common.models import VTCommandLineArgsBase
from common.api import virtual_sfv_pb2
from common.api import solar_field_common_pb2
from common.api import common_models_pb2
import google.protobuf.json_format as json_format
from google.protobuf import wrappers_pb2
from user_calculation import calculate as user_calc
from user_calculation import plot as user_plot
import pandas as pd
from math import isnan
from google.protobuf import struct_pb2

class VirtualSfvRunner(ABC):
    def __init__(self, metadata_file: str):
        self.metadata = self.get_metadata(metadata_file)

    def get_metadata(self, metadata_file: str) -> virtual_sfv_pb2.VirtualSfvModule:
        with open(metadata_file, 'r', encoding='utf-8') as f:
            message = virtual_sfv_pb2.VirtualSfvModule()
            json_format.Parse(f.read(), message)
            return message


    def calculate(self, vtCommandLineArgs: VTCommandLineArgsBase, query_time_utc: datetime, margin: timedelta, inputs_from_solar_focus: pd.DataFrame = None) -> pd.DataFrame:
        inputDf = sofoc.solar_field_to_df(self.metadata.input_tags, inputs_from_solar_focus)
        inputUpdateTime = sofoc.solar_field_to_update_time(self.metadata.input_tags, inputs_from_solar_focus)
        res = user_calc(query_time_utc, margin, inputDf, inputUpdateTime)
        resSolarFieldDto = self.__df_to_solar_field__(res)
        self.validate_res(resSolarFieldDto, inputs_from_solar_focus)
        if vtCommandLineArgs.debug:
            user_plot(res)
        return resSolarFieldDto

    def validate_res(self, res: solar_field_common_pb2.SolarFieldDto, inputs_from_solar_focus: solar_field_common_pb2.SolarFieldDto = None):
        """
        Verify that result of this virtual sfv match the defines outputs
        """
        if len(res.data) != len(self.metadata.output_tags):
            raise Exception(f"Output result does not match output tags size")

        if inputs_from_solar_focus is None:
            return

        for i in range(0, len(self.metadata.output_tags)):
            output_md_name = self.metadata.output_tags[i].name
            output_name = res.data[i].tag_id.virtual_tag_identifier.output_name
            if (output_md_name != output_name):
                raise Exception(f"Output name does not match output md name")

        layout_size = len(inputs_from_solar_focus.data[0].nullable_data)
        for tag in res.data:
            if len(tag.nullable_data) != layout_size:
                raise Exception(f"Output result does not match layout size")

    def execute(self, vtCommandLineArgs: VTCommandLineArgsBase, query_time_utc: datetime, margin: timedelta) -> solar_field_common_pb2.SolarFieldDto:
        """
        This is the execution call for this VT calculation flow
        """
        query: virtual_sfv_pb2.SfvQuery = sofoc.build_query(query_time_utc, margin,self.metadata.input_tags, self.metadata.calculation_method)
        if len(query.tags_descriptions) > 0:
            res: solar_field_common_pb2.SolarFieldDto = sofoc.get_data(query, vtCommandLineArgs)
            return self.calculate(vtCommandLineArgs = vtCommandLineArgs, query_time_utc = query.query_time_utc.ToDatetime(), margin = query.margin.ToTimedelta(), inputs_from_solar_focus = res)
        else:
            return self.calculate(vtCommandLineArgs = vtCommandLineArgs, query_time_utc = query.query_time_utc.ToDatetime(), margin = query.margin.ToTimedelta())

    def __number_to_proto_float__(self, number) -> wrappers_pb2.FloatValue:
        fv = wrappers_pb2.FloatValue()
        fv.value = number
        return fv

    def __df_to_solar_field__(self, dF: pd.DataFrame) -> solar_field_common_pb2.SolarFieldDto:
        output_res = solar_field_common_pb2.SolarFieldDto()
        for output_tag in self.metadata.output_tags:
            tag_data = solar_field_common_pb2.TagData()
            data = dF[output_tag.name]
            data = data.apply(self.__number_to_proto_float__)
            tag_id = common_models_pb2.VirtualTagIdentifier()
            tag_data.tag_id.virtual_tag_identifier.module_name = self.metadata.name
            tag_data.tag_id.virtual_tag_identifier.output_name = output_tag.name
            tag_data.tag_id.tag_type = common_models_pb2.IdentifierTypes.VIRTUAL
            tag_data.nullable_data.extend(data)
            output_res.data.append(tag_data)
        return output_res


import pymongo
import datetime
import pandas as pd

class MongoReader(object):
    def __init__(self, connection_string, db_name="Archive"):
        self.connection_string = connection_string
        self.client = pymongo.MongoClient(self.connection_string)
        self.db = self.client[db_name]

    def get_collection(self, collection_name):
        return self.db[collection_name]

    def get_cursor_for_collection(self, collection_name, filter={}, projection=None, skip=0, limit=5000,
                                  sort=None, batch_size=100):
        collection = self.get_collection(collection_name)
        return collection.find(filter, limit=limit, skip=skip, sort=sort, batch_size=batch_size)

    def get_document(self, collection_name, filter, sort):
        cursor = self.get_cursor_for_collection(collection_name, filter, sort=sort)
        for doc in cursor.limit(1):
            return doc

    def get_document_as_series(self, collection_name, filter, sort, idx_field: str, data_field: str, header="value") -> pd.DataFrame:
        doc = self.get_document(collection_name, filter, sort)
        return self.__convert_doc_to_series(doc[data_field], header)

    def __convert_doc_to_series(self, doc, header="value") -> pd.DataFrame:
        series = pd.Series(doc)
        df = pd.DataFrame({'id': series.index, header: series.values})
        df.id = df.id.map(int)
        return df





#!/usr/bin/python3
import argparse
import json
import sys
import os
from common.models import VTCommandLineArgsBase
from common.api import virtual_trends_pb2_grpc
from common.api import virtual_trends_pb2
from common.api import common_models_pb2
from common import utils
from common import models
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
import grpc
import pandas as pd
from typing import List, Dict
from datetime import datetime, timedelta
from google.protobuf.json_format import MessageToJson

TIME_FORMAT = "%Y-%m-%d %H:%M:%S"


def build_query(start_utc: datetime,
                end_utc: datetime,
                defined_inputs: List[common_models_pb2.VirtualModuleInputTag],
                calculation_method: virtual_trends_pb2.CalculationMethod,
                sample_frequency:google_dot_protobuf_dot_duration__pb2.Duration ) -> virtual_trends_pb2.TimeSeriesQueryInput:
    res = virtual_trends_pb2.TimeSeriesQueryInput( calculation_method=calculation_method, sample_frequency=sample_frequency)
    for input in defined_inputs:
        res.trends_identifier.append(input.raw_id)
    if end_utc < start_utc:
        raise Exception("Query params error: end time - {0} is smaller than start time - {1}".format(end_utc, start_utc))
    res.start_date_utc.FromDatetime(start_utc)
    res.end_date_utc.FromDatetime(end_utc)
    return res

def get_data(defined_inputs: List[common_models_pb2.VirtualModuleInputTag], query: virtual_trends_pb2.TimeSeriesQueryInput, args: VTCommandLineArgsBase) -> pd.DataFrame:
    channel = utils.get_grpc_channel_according_to_args(args)
    client = virtual_trends_pb2_grpc.VirtualTrendsGrpcServiceStub(channel)
    res = client.GetData(query, metadata=[('x-api-key', f'{args.api_access_key}')])
    return convert_trends_res_to_df(defined_inputs, res)

def convert_trends_res_to_df(defined_inputs: List[common_models_pb2.VirtualModuleInputTag], trends_res: virtual_trends_pb2.TimeSeriesQueryResult) -> pd.DataFrame:
    problematics = trends_res.problematic_tags
    if len(defined_inputs) == 0:
        return pd.DataFrame()
    if len(problematics) > 0:
        raise Exception(
            f"Didn't get data for {len(problematics)} tags, reasons: {problematics}")
    df = pd.concat(map(lambda x: convert_sf_series_to_pandas(x.time_series), trends_res.results), axis=1, keys=map(
        lambda x: x.trend_id.SerializeToString(), trends_res.results))
    df = df.rename(lambda i: next(
            x for x in defined_inputs if x.raw_id.SerializeToString() == i).variable_name, axis='columns')
    return df


def convert_sf_series_to_pandas(data: List) -> pd.Series:
    index = pd.DatetimeIndex([x.time.ToDatetime() for x in data])
    vals = [x.value for x in data]
    return pd.Series(vals, index=index)

def plot(df: pd.DataFrame):
     import matplotlib.pyplot as plt
     df.plot()
     plt.legend(loc='best')
     plt.pause(60)

def parse_input(cmd_line_args) -> (VTCommandLineArgsBase, datetime, datetime ):
    (base_cmd_line_args, extra_args) = utils.parse_input(cmd_line_args)
    parser = argparse.ArgumentParser(
        description='Input parameters to run virtual trend')
    parser.add_argument('--start-utc', action='store', required='--metadata' not in cmd_line_args,
                        dest='start',
                        help='Start time of the query in UTC, format YYYY-MM-DD hh:mm:ss')
    parser.add_argument('--end-utc', action='store', required='--metadata' not in cmd_line_args,
                        dest='end',
                        help='End time of the query in UTC, format YYYY-MM-DD hh:mm:ss')
    args = parser.parse_args(extra_args)
    if args.start is not None and args.end is not None:
        start_utc = datetime.strptime(args.start, TIME_FORMAT)
        end_utc = datetime.strptime(args.end, TIME_FORMAT)
    else:
        start_utc = None
        end_utc = None
    return base_cmd_line_args,start_utc, end_utc



#!/usr/bin/python3
from abc import ABC, abstractmethod
from datetime import datetime, timedelta
from . import solar_focus_tables_lib as sofoc
from common.models import VTCommandLineArgsBase
from common.api import virtual_tables_pb2
from common.api import solar_field_pb2
from common.api import solar_field_common_pb2
from common.api import virtual_sfv_pb2
from common.api import virtual_trends_pb2
from common.api import common_models_pb2
from virtual_sfv import solar_focus_sfv_lib as virtual_sfv_lib
from virtual_trend import solar_focus_trends_lib as virtual_trends_lib
import google.protobuf.json_format as json_format
from google.protobuf import wrappers_pb2
from user_calculation import calculate as user_calc
import pandas as pd
from math import isnan
from google.protobuf import struct_pb2
from typing import Dict, List

class VirtualTablesRunner(ABC):
    def __init__(self, metadata_file: str):
        self.metadata = self.get_metadata(metadata_file)

    def get_metadata(self, metadata_file: str) -> virtual_tables_pb2.VirtualTablesModule:
        import os
        cwd = os.getcwd()
        metadata_file_path = os.path.join(cwd,metadata_file)
        with open(metadata_file_path, 'r', encoding='utf-8') as f:
            message = virtual_tables_pb2.VirtualTablesModule()
            json_format.Parse(f.read(), message)
            return message

    def build_query(self, sfv_query_time_utc: datetime, sfv_margin: timedelta, trends_start_utc: datetime, trends_end_utc: datetime) -> virtual_tables_pb2.VirtualTablesQuery:
        sfv_query = virtual_sfv_lib.build_query(sfv_query_time_utc, sfv_margin,self.metadata.input_sfv_tags, self.metadata.sfv_calculation_method)
        trends_query = virtual_trends_lib.build_query(trends_start_utc, trends_end_utc,self.metadata.input_trends, self.metadata.trends_calculation_method, self.metadata.sample_frequency)
        res = virtual_tables_pb2.VirtualTablesQuery(sfv_query = sfv_query, trends_query = trends_query)
        return res
    
    def calculate(self, sfv_query_time_utc: datetime, sfv_margin: timedelta, trends_start_utc: datetime, trends_end_utc: datetime, inputs_from_solar_focus: virtual_tables_pb2.VirtualTablesInputsResult = None) -> Dict[str, pd.DataFrame]:
        if inputs_from_solar_focus is None:
            res = user_calc(sfv_query_time_utc = sfv_query_time_utc, sfv_margin = sfv_margin, trends_start_utc = trends_start_utc, trends_end_utc = trends_end_utc)
        else: 
            input_sfv_df = virtual_sfv_lib.solar_field_to_df(self.metadata.input_sfv_tags, inputs_from_solar_focus.sfv)
            input_trends_df = virtual_trends_lib.convert_trends_res_to_df(self.metadata.input_trends, inputs_from_solar_focus.trends)
            res = user_calc(sfv_query_time_utc = sfv_query_time_utc, sfv_margin = sfv_margin, trends_start_utc = trends_start_utc, trends_end_utc = trends_end_utc, trends_inputs_from_solar_focus = input_trends_df, sfv_inputs_from_solar_focus = input_sfv_df)
        self.validate_res(res)
        return res

    def validate_res(self, res: Dict[str, pd.DataFrame]):
        """
        Verify that result of this virtual sfv match the defines outputs
        """
        if len(res) != len(self.metadata.output_tables):
            raise Exception(f"Output result does not match output tags size")

        for output_md in self.metadata.output_tables:
            if (output_md.name not in res.keys()):
                raise Exception(f"Output name does not match output md name")
            

    def execute(self, vtCommandLineArgs: VTCommandLineArgsBase, sfv_query_time_utc, sfv_margin, trends_start_utc, trends_end_utc) -> List[pd.DataFrame]:
        """
        This is the execution call for this VT calculation flow
        """
        query: virtual_tables_pb2.VirtualTablesQuery = self.build_query(sfv_query_time_utc, sfv_margin, trends_start_utc, trends_end_utc)
        if len(query.sfv_query.tags_descriptions) > 0 or len(query.trends_query.trends_identifier) > 0: 
            res: virtual_tables_pb2.VirtualTablesInputsResult = sofoc.get_data(query, vtCommandLineArgs)
            return self.calculate(sfv_query_time_utc = query.sfv_query.query_time_utc.ToDatetime(), sfv_margin= query.sfv_query.margin.ToTimedelta(), trends_start_utc = query.trends_query.start_date_utc.ToDatetime(), trends_end_utc = query.trends_query.end_date_utc.ToDatetime(), inputs_from_solar_focus = res)
        else:
            return self.calculate(sfv_query_time_utc = query.sfv_query.query_time_utc.ToDatetime(), sfv_margin= query.sfv_query.margin.ToTimedelta(), trends_start_utc = query.trends_query.start_date_utc.ToDatetime(), trends_end_utc = query.trends_query.end_date_utc.ToDatetime())



#!/usr/bin/python3
from common import models
import grpc
from datetime import timedelta
import re
import argparse
import os
timedelta_regex = re.compile(r'((?P<hours>\d{2})):((?P<minutes>\d{2})):((?P<seconds>\d{2}))')
MAX_MESSAGE_LENGTH = 100 * 1024 * 1024

def get_grpc_channel_according_to_args(args: models.VTCommandLineArgsBase) -> grpc.Channel:
    if args.insecure:
        channel = grpc.insecure_channel(args.execution_url, options = [
        ('grpc.max_send_message_length', MAX_MESSAGE_LENGTH),
        ('grpc.max_receive_message_length', MAX_MESSAGE_LENGTH),
        ])
    else:
        credentials = grpc.ssl_channel_credentials()
        channel = grpc.secure_channel(args.execution_url, credentials,  options = [
        ('grpc.max_send_message_length', MAX_MESSAGE_LENGTH),
        ('grpc.max_receive_message_length', MAX_MESSAGE_LENGTH),
        ])
    return channel

def parse_time(time_str) -> timedelta:
    parts = timedelta_regex.match(time_str)
    if not parts:
        return
    parts = parts.groupdict()
    time_params = {}
    for (name, param) in parts.items():
        if param:
            time_params[name] = int(param)
    return timedelta(**time_params)

def parse_input(cmd_line_args) -> (models.VTCommandLineArgsBase, list):
    parser = argparse.ArgumentParser(
        description='Input parameters to run virtual data module')
    parser.add_argument('--api-access-key', action='store', required='--debug' in cmd_line_args,
                        dest='api_access_key',
                        help='virtual trends api aceess key')
    parser.add_argument('--insecure', action='store_true',
                        dest='insecure',
                        help='do not verfiy https certificate - used for SF execution')
    parser.add_argument('--cert', action='store',
                        dest='cert', default='cert.pem',
                        help='path to root CA relative to entry point file')
    parser.add_argument('--execution-url', dest='execution_url',
                        action='store', default='solarfocus.bseinc.com')
    parser.add_argument('--debug', dest='debug', action='store_true')
    parser.add_argument('--metadata-file', dest='metadata_file',
                        action='store', default='metadata.json')
    parser.add_argument('--output-filename', action='store', required='--debug' not in cmd_line_args and '--metadata' not in cmd_line_args,
                        dest='output_filename',
                        help='Output filename to extract')
    args = parser.parse_known_args(cmd_line_args)
    known_args = args[0]
    execution_url = known_args.execution_url
    debug = known_args.debug if known_args.debug is not None else False
    insecure = known_args.insecure if known_args.insecure is not None else False
    metadata_file = known_args.metadata_file
    return (models.VTCommandLineArgsBase(metadata_file, execution_url, debug,
                             known_args.output_filename, known_args.api_access_key, insecure, known_args.cert), args[1])


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from . import tags_metadata_pb2 as tags__metadata__pb2


class TagsMetadataGrpcServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetUnfilteredAllMetaData = channel.unary_unary(
                '/Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService/GetUnfilteredAllMetaData',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=tags__metadata__pb2.TagsMetadataDto.FromString,
                )
        self.GetUnfilteredSiteMetaData = channel.unary_unary(
                '/Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService/GetUnfilteredSiteMetaData',
                request_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
                response_deserializer=tags__metadata__pb2.TagsMetadataDto.FromString,
                )
        self.GetSiteMetaData = channel.unary_unary(
                '/Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService/GetSiteMetaData',
                request_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
                response_deserializer=tags__metadata__pb2.TagsMetadataDto.FromString,
                )
        self.UpdateTagMetaData = channel.unary_unary(
                '/Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService/UpdateTagMetaData',
                request_serializer=tags__metadata__pb2.TagDto.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )


class TagsMetadataGrpcServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetUnfilteredAllMetaData(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetUnfilteredSiteMetaData(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSiteMetaData(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateTagMetaData(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TagsMetadataGrpcServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetUnfilteredAllMetaData': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUnfilteredAllMetaData,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=tags__metadata__pb2.TagsMetadataDto.SerializeToString,
            ),
            'GetUnfilteredSiteMetaData': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUnfilteredSiteMetaData,
                    request_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
                    response_serializer=tags__metadata__pb2.TagsMetadataDto.SerializeToString,
            ),
            'GetSiteMetaData': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSiteMetaData,
                    request_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
                    response_serializer=tags__metadata__pb2.TagsMetadataDto.SerializeToString,
            ),
            'UpdateTagMetaData': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateTagMetaData,
                    request_deserializer=tags__metadata__pb2.TagDto.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class TagsMetadataGrpcService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetUnfilteredAllMetaData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService/GetUnfilteredAllMetaData',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            tags__metadata__pb2.TagsMetadataDto.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetUnfilteredSiteMetaData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService/GetUnfilteredSiteMetaData',
            google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
            tags__metadata__pb2.TagsMetadataDto.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSiteMetaData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService/GetSiteMetaData',
            google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
            tags__metadata__pb2.TagsMetadataDto.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateTagMetaData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService/UpdateTagMetaData',
            tags__metadata__pb2.TagDto.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
